/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_IDSTR(id, "@(#)$Id: t-batv-0.c,v 1.3 2005/02/28 23:46:16 ca Exp $")

#include "sm/error.h"
#include "sm/mta.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-prt.h"
#include "sm/batv.h"
#include "sm/batvcnf.h"
#define SM_BATVCNFDEF	1
#include "sm/batvcnfdef.h"
#include "sm/io.h"
#include "sm/test.h"

#if SM_USE_BATV
static int Verbose;
static batv_cnf_T batv_cnf;

static sm_ret_T
read_batv_cnf(const char *path)
{
	int err;
	FILE *fp;
	sm_conf_T *smc;

	fp = NULL;
	sm_memzero(&batv_cnf, sizeof(batv_cnf));
	smc = sm_conf_new(path);
	if (smc == NULL)
	{
		err = errno;

		sm_io_fprintf(smioerr,
			"sev=ERROR, func=sm_readconf, file=%s, sm_conf_new=NULL, errno=%d\n",
			path, err);
		return ENOMEM;
	}
	err = sm_conf_read_FILE(smc, path, fp);
	if (err != 0)
	{
		sm_prt_conferr(path, smc, err);
		sm_conf_destroy(smc);
		smc = NULL;
		return err;
	}

	err = sm_conf_scan(smc, batv_defs, 0, &batv_cnf);
	if (err != 0)
	{
		sm_prt_conferr(path, smc, err);
		return err;
	}
	return SM_SUCCESS;
}

static sm_ret_T
test(const char *address)
{
	sm_ret_T ret;
/*
	batv_T batv_sig;
*/
	sm_str_P addr, batvaddr;

	addr = batvaddr = NULL;
	addr = sm_str_new(NULL, MAXADDRLEN, MAXADDRLEN);
	if (addr == NULL)
		goto enomem;
	batvaddr = sm_str_new(NULL, MAXADDRLEN, MAXADDRLEN);
	if (batvaddr == NULL)
		goto enomem;

	ret = sm_str_scat(addr, address);
	SM_TEST(ret == SM_SUCCESS);
	if (sm_is_err(ret))
		goto error;
	ret = sm_batv_saddr(addr, 1, &batv_cnf, batvaddr);
	SM_TEST(ret == SM_SUCCESS);
	if (sm_is_err(ret))
		goto error;
	sm_io_fprintf(smioerr, "address='%S'\nsigned='%S'\n",
		addr, batvaddr);
	ret = sm_batv_vrfy(batvaddr, &batv_cnf);
	SM_TEST(ret == SM_SUCCESS);
	if (sm_is_err(ret))
		goto error;

	return SM_SUCCESS;

  enomem:
	ret = sm_err_temp(ENOMEM);
  error:
	SM_STR_FREE(addr);
	SM_STR_FREE(batvaddr);
	return ret;
}

int
main(int argc, char *argv[])
{
	int c, r;
	char *prg, *cnf;

	opterr = 0;
	Verbose = 0;
	prg = argv[0];
	cnf = NULL;
	while ((c = getopt(argc, argv, "f:V")) != -1)
	{
		switch (c)
		{
		  case 'f':
			cnf = optarg;
			r = read_batv_cnf(cnf);
			SM_TEST(r == SM_SUCCESS);
			break;
		  case 'V':
			++Verbose;
			break;
		  default:
			exit(1);
		}
	}
	sm_test_begin(argc, argv, "test batv 0");
	argc -= optind;
	argv += optind;
	SM_TEST(cnf != NULL);
	if (argc > 0 && cnf != NULL)
	{
		for (r = 0; r < argc; r++)
		{
			test(argv[r]);
		}
	}
	return sm_test_end();
}
#else /* SM_USE_BATV */
int
main(int argc, char *argv[])
{
	sm_io_fprintf(smioerr, "SM_USE_BATV not available\n");
	return 0;
}
#endif /* SM_USE_BATV */
