/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: smbatvsign1.c,v 1.1 2005/01/31 01:59:13 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/time.h"
#include "sm/batv.h"
#include "sm/batvcnf.h"

#if SM_USE_BATV
/*
**  SM_BATV_SIGN1 -- create a BATV signature
**	wrapper for sm_batv_sign() to calculate timestamp ("days")
**
**	Parameters:
**		address -- address to sign
**		key -- BATV key to use
**		batv_cnf -- BATV configuration
**		batv_sig -- BATV signature (output)
**
**	Returns:
**		usual return code
*/

sm_ret_T
sm_batv_sign1(const char *address, uint key, batv_cnf_P batv_cnf, batv_P batv_sig)
{
	time_t now;	/* should this be passed in? */
	uint days;

	SM_REQUIRE(address != NULL);
	SM_REQUIRE(batv_cnf != NULL);
	SM_REQUIRE(batv_sig != NULL);

	now = time(NULL);

	/* simple way to calculate days since epoch, ignores leap sec etc */
	days = now / (60 * 60 * 24) + batv_cnf->batvcnf_timeout;

	return sm_batv_sign(address, days, key, batv_cnf, batv_sig);
}
#endif /* SM_USE_BATV */
